%% CSC321 Tutorial 1 Matlab Demo 
% Written by Yue Li, 01/2012
% Adapted from matlab_tutorial.m by Inmar Givoni (01/2010)


%% General stuff (Note how %% is used to have a division line to ease internal documentation)

disp('CSC321 Matlab Tutorial 1');

% matlab recognizes the files in the directory it is pointing at, first thing
% to do is make sure you are in the right place.

pwd
cd('~/Desktop/CSC321/Tutorials/T1/matlab_code/') % change to the directory where the scripts are saved
ls                  % like unix
load('data.mat');   % loads a mat file
who                 % who are my variables?
whos                % who are my variables?
clear               % clear all variables (caution!)
                    % (often done at the begining)
                    % can be used as function e.g. clear(x)
clc                 % clear command window
help plot           % help X will help!

%% looking at data
load('data.mat'); % loads a mat file

plot(test_x, test_y);       % default is line, no data points. 
plot(test_x, test_y,'-r.'); % show dot markers for the data points! 
                            % dot almost always needed (do this in your assignments)!
xlabel('X LABEL');
ylabel('Y LABEL','fontsize',12);
title('My Graph');

% plot 
figure;                     % to start another figure. 
plot(test_x, test_y,'-r.', test_x, 2*test_y+0.5,'-b.');
legend('data set 1','data set 2');

% alternatively use hold on, hold off commands
figure;                     % to start another figure. 
plot(test_x, test_y,'-r.');
hold on;
plot(test_x, 2*test_y+0.5,'-b.');
legend('data set 1','data set 2');
hold off;


%% for, while, if, elseif

% remember to close with 'end'. 

i = rand(10,1);
if sum(i) > 5
    disp('Greater than 5');
elseif sum(i)> 2.5
    disp('Greater than 2.5');
else
    disp('Less than 2.5');
end




%% Accessing and manipulating matrices

x = rand(100000,1);
length(x)           % only row number
size(x)             % gives both row and col number
x = x';             % transpose
size(x)             
y = rand(20,30);
size(y)
size(y,2)

x(1:10)
x(1:3:12)
x(1:10)*3
x(1:10) = [];       % remove elements
size(x)
x(1:10).*x(1:10)    % element by element


%% Performance comparison of different numerical operations that produce
% the same results

tic                 % record start time
sum(x.*x)           % explicitly multiply and sum
toc                 % record end time

tic
x*x'                % as matrix multiplication (same result as above)
toc

% standard for loop way - avoid if you want to write efficient matlab code
tic
tot = 0;
for i=1:length(x)       % note the usage
   tot = tot+x(i)^2;
end
tot
toc

%% repmat

x=3;
y=1:10;

% multiply every element of y by x
x*y

x = [3,4,5]';
y= [1:10;2:11;3:12];

% .* multiply every column of y by x without a for loop
z = y .* repmat(x,1,size(y,2));

disp(x);disp(y);disp(z);

%% Vectorizing (more on Tutorial 2)

x = rand(10000,5); % input of 10000 cases; each case has 5 variables
w1 = [1,3,-4,5,6]; % the parameter vector you learned for class 1
                   % (i.e., parameters from an already trained model)
w0 = [2,5,5,2,1];  % the parameter vector you learned for class 0
                   % (i.e., parameters from an already trained model)
                   
% calculate the p(y=1|x) for every x
tic
for i=1:size(x,1)
    temp0 = 0;
    temp1 = 0;
    for j=1:size(x,2)
        temp0 = temp0+ w0(j)*x(i,j);
        temp1 = temp1+ w1(j)*x(i,j);
    end
    p_y0(i) = exp(-temp0);
    p_y1(i) = exp(-temp1);
    sum = p_y0(i)+p_y1(i);
    p_y0(i) = p_y0(i)/sum;  % normalization s.t. p_y0(i) + p_y1(i) = 1
    p_y1(i) = p_y1(i)/sum;
end
toc

% Matrix multiplication obviates the above iteration loops
% and is much more efficient
tic
p2_y1 = exp(-x*w1')./(exp(-x*w1')+exp(-x*w0'));
p2_y0 = exp(-x*w0')./(exp(-x*w1')+exp(-x*w0'));
toc

% sanity checkf
any(p2_y0~=p_y0') % note ~= (rather than !=) indicates not equal 

p2_y0+p2_y1 % didn't really need the second part...




%% Functions:
% 1 function per file (that can be called from outside), file name is function name
% (even if you define it differently!)
[val,grad] = calc_val_gradient([1,2]) % let's see what's in there


 
%% Check grad:
% to verify that your derivative is correct use check grad:
checkgrad('calc_val_gradient', randn(2, 1), 0.001) %func name, point, tolerance, [other inputs]


%% Misc
% energy surface from 'doc surf'
[X,Y,Z] = peaks(30);
surfc(X,Y,Z)
colormap hsv
axis([-3 3 -3 3 -10 5])


% display digit image
load('./digit2.mat');
subplot(1,5,1);
mnistdisp(D(1,:)');
subplot(1,5,2);
mnistdisp(D(2,:)');
subplot(1,5,3);
mnistdisp(D(4,:)');
subplot(1,5,4);
mnistdisp(D(5,:)');
subplot(1,5,5);
mnistdisp(D(1,:)');

